//
//  GameController.swift
//  Anagrams
//
//  Created by Caroline Begbie on 12/04/2015.
//  Copyright (c) 2015 Caroline. All rights reserved.
//

import Foundation
import UIKit

class GameController {
  var gameView: UIView!
  var level: Level!
  
  private var tiles = [TileView]()
  private var targets = [TargetView]()
  
  var hud: HUDView!
  
  //stopwatch variables
  private var secondsLeft: Int = 0
  private var timer: NSTimer?
  
  private var data = GameData()
  
  init() {
  }
  
  func dealRandomAnagram () {
    //1
    assert(level.anagrams.count > 0, "no level loaded")
    
    //2
    let randomIndex = randomNumber(minX:0, maxX:UInt32(level.anagrams.count-1))
    let anagramPair = level.anagrams[randomIndex]
    
    //3
    let anagram1 = anagramPair[0] as! String
    let anagram2 = anagramPair[1] as! String
    
    //4
    let anagram1length = count(anagram1)
    let anagram2length = count(anagram2)
    
    //5
    println("phrase1[\(anagram1length)]: \(anagram1)")
    println("phrase2[\(anagram2length)]: \(anagram2)")
    
    //calculate the tile size
    let tileSide = ceil(ScreenWidth * 0.9 / CGFloat(max(anagram1length, anagram2length))) - TileMargin
    
    //get the left margin for first tile
    var xOffset = (ScreenWidth - CGFloat(max(anagram1length, anagram2length)) * (tileSide + TileMargin)) / 2.0
    
    //adjust for tile center (instead of the tile's origin)
    xOffset += tileSide / 2.0
    
    //initialize target list
    targets = []
    
    //create targets
    for (index, letter) in enumerate(anagram2) {
      if letter != " " {
        let target = TargetView(letter: letter, sideLength: tileSide)
        target.center = CGPointMake(xOffset + CGFloat(index)*(tileSide + TileMargin), ScreenHeight/4)
        
        gameView.addSubview(target)
        targets.append(target)
      }
    }
    
    //1 initialize tile list
    tiles = []
    
    //2 create tiles
    for (index, letter) in enumerate(anagram1) {
      //3
      if letter != " " {
        let tile = TileView(letter: letter, sideLength: tileSide)
        tile.center = CGPointMake(xOffset + CGFloat(index)*(tileSide + TileMargin), ScreenHeight/4*3)
        
        tile.randomize()
        tile.dragDelegate = self
        
        //4
        gameView.addSubview(tile)
        tiles.append(tile)
      }
    }
    
    //start the timer
    self.startStopwatch()
  }
  
  func placeTile(tileView: TileView, targetView: TargetView) {
    //1
    targetView.isMatched = true
    tileView.isMatched = true
    
    //2
    tileView.userInteractionEnabled = false
    
    //3
    UIView.animateWithDuration(0.35,
      delay:0.00,
      options:UIViewAnimationOptions.CurveEaseOut,
      //4
      animations: {
        tileView.center = targetView.center
        tileView.transform = CGAffineTransformIdentity
      },
      //5
      completion: {
        (value:Bool) in
        targetView.hidden = true
    })
  }
  
  
  
  func checkForSuccess() {
    for targetView in targets {
      //no success, bail out
      if !targetView.isMatched {
        return
      }
    }
    println("Game Over!")
    
    //stop the stopwatch
    self.stopStopwatch()
  }

  func startStopwatch() {
    //initialize the timer HUD
    secondsLeft = level.timeToSolve
    hud.stopwatch.setSeconds(secondsLeft)
    
    //schedule a new timer
    timer = NSTimer.scheduledTimerWithTimeInterval(1.0, target: self, selector:"tick:", userInfo: nil, repeats: true)
  }
  
  func stopStopwatch() {
    timer?.invalidate()
    timer = nil
  }
  
  @objc func tick(timer: NSTimer) {
    secondsLeft--
    hud.stopwatch.setSeconds(secondsLeft)
    if secondsLeft == 0 {
      self.stopStopwatch()
    }
  }
}

extension GameController:TileDragDelegateProtocol {
  //a tile was dragged, check if matches a target
  func tileView(tileView: TileView, didDragToPoint point: CGPoint) {
    var targetView: TargetView?
    for tv in targets {
      if tv.frame.contains(point) && !tv.isMatched {
        targetView = tv
        break
      }
    }
    
    //1 check if target was found
    if let targetView = targetView {
      
      //2 check if letter matches
      if targetView.letter == tileView.letter {
        
        //3
        self.placeTile(tileView, targetView: targetView)

        //more stuff to do on success here
        
        //give points
        data.points += level.pointsPerTile
        hud.gamePoints.setValue(data.points, duration: 0.5)
        
        //check for finished game
        self.checkForSuccess()
      
      } else {
        
        //4
        //1
        tileView.randomize()
        
        //2
        UIView.animateWithDuration(0.35,
          delay:0.00,
          options:UIViewAnimationOptions.CurveEaseOut,
          animations: {
            tileView.center = CGPointMake(tileView.center.x + CGFloat(randomNumber(minX:0, maxX:40)-20),
              tileView.center.y + CGFloat(randomNumber(minX:20, maxX:30)))
          },
          completion: nil)
        
        //more stuff to do on failure here
        
        //take out points
        data.points -= level.pointsPerTile/2
        hud.gamePoints.setValue(data.points, duration: 0.25)
      }
    }
    
  }
  

}
